#! /bin/bash

#   Copyright (c) 1998  Martin Schulze <joey@debian.org>
#   Slightly modified by Luis Francisco Gonzalez <luisgh@debian.org>
#   Path lists implemented with Bash-arrays 
#	by Ivan Zakharyaschev <imz@altlinux.ru> November 2001
#   Clean up by ALT Linux Team <devel@altlinux.ru> April 2002
#   imz@altlinux.ru, 2003 Feb 8: 
#   - more quotes (to prevent expansion in wrong places like the call of tr);
#   - accept ./* as file-URL, too.

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

PROG="${0##*/}"

#set -x

########################
# Load program lists.  #
########################

# Why not to make a stdBash library? Not to copy such things everywhere.
SourceIfExists()
{
	local f="$1"
	shift
	[ -f "$f" ] && . "$f" "$@"
}

Error()
{
    toReturn="$1"
    cat >&2 # pass the msg to stderr
    read -p $"Press any key to continue.." -s -n1
    exit "$toReturn"
}

SourceIfExists /etc/urlview/url_handlers
[ -n "$HOME" ] && SourceIfExists "$HOME"/.etc/urlview/url_handlers

######################################
# Process the data and run a handler #
######################################

TAG=
HANDLER=

# Accepts a list of available handler-scripts:tag pairs as parameters.
# Assigns the first executable values one to HANDLER & TAG,
# possibly adding special constructs to HANDLER.
function getprg()
{
	local ele tag prog

	for ele in "$@"; do
		tag="${ele##*:}"
		prog="${ele%%:*}"
		# Check that the executable is installed on this system:
		if [[ ! -x "${prog%% *}" ]]; then
		    continue
		fi
		case "$tag" in
		    PW)
			if [ -n "$DISPLAY" ]; then
			    TAG=P
			    HANDLER="$prog"
			    return 0
			fi
			;;
		    XW)
			if [ -n "$DISPLAY" ]; then
			    TAG=X
			    HANDLER="$prog"
			    return 0
			fi
			;;
		    XT)
			if [ -n "$DISPLAY" -a -x "$XTERM" ]; then
			    TAG=X
			    HANDLER="$XTERM -e $prog"
			    return 0
			else
			    HANDLER="$prog"
			    return 0
			fi
			;;
		    VT)
			HANDLER="$prog"
			return 0
			;;
		esac
	done
}

function setType()
{
    type="$(echo "$1" | tr '[:upper:]' '[:lower:]')"
}

url="$1"; shift

case "$url" in
    /*)
	setType file
	url="$type://$url"
	;;
    ./*)
	setType file
	url="$type://$(pwd)/${url##./}"
	;;
    *:*) 
	setType "${url%%:*}"
	;;
    *.*) 
	setType "${url%%.*}"
	case "$type" in
	    www|web) setType http ;;
	esac
	url="$type://$url"
	;;
    *)
	setType http
	;;
esac

if ! eval '(( "${#'"$type"'_prgs[@]}" ))'; then
    {
	printf $"%s: unknown URL type \"%s\"! "	"$PROG" "$type" 
	printf $"Please report URL and viewer to %s\n" \
	    "http://bugs.altlinux.ru/" 
    } | Error 1
fi

eval 'getprg "${'"$type"'_prgs[@]}"'

if [ -z "$HANDLER" ]; then
    {
	printf $"%s: no handler found for URL \"%s\"! "	"$PROG" "$url" 
	printf $"You may report URL and viewer to %s\n" \
	    "http://bugs.altlinux.ru/" 
    } | Error 1
fi

# type-special hooks: some processing can be appended here
case "$type" in
	mailto)
		url="${url#mailto:}"
		url="${url#//}"
		;;
esac

if [ -n "$HANDLER" ]; then
	case "$TAG" in
	    P)	nohup $HANDLER "$url" >/dev/null 2>&1 & ;;
	    X)	$HANDLER "$url" 2>/dev/null & ;;
	    *)  $HANDLER "$url" ;;
	esac
fi
